@@ -4,58 +4,70 @@
 
     Sphinx deprecation classes and utilities.
 
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import sys
 import warnings
 from importlib import import_module
-from typing import Any, Dict
-
-if False:
-    # For type annotation
-    from typing import Type  # for python3.5.1
+from typing import Any, Dict, Type
 
 
 class RemovedInSphinx40Warning(DeprecationWarning):
     pass
 
 
-class RemovedInSphinx50Warning(PendingDeprecationWarning):
+class RemovedInSphinx50Warning(DeprecationWarning):
     pass
 
 
-RemovedInNextVersionWarning = RemovedInSphinx40Warning
+class RemovedInSphinx60Warning(PendingDeprecationWarning):
+    pass
 
 
-def deprecated_alias(modname: str, objects: Dict, warning: "Type[Warning]") -> None:
+RemovedInNextVersionWarning = RemovedInSphinx50Warning
+
+
+def deprecated_alias(modname: str, objects: Dict[str, object],
+                     warning: Type[Warning], names: Dict[str, str] = {}) -> None:
     module = import_module(modname)
-    sys.modules[modname] = _ModuleWrapper(module, modname, objects, warning)  # type: ignore
+    sys.modules[modname] = _ModuleWrapper(  # type: ignore
+        module, modname, objects, warning, names)
 
 
 class _ModuleWrapper:
-    def __init__(self, module: Any, modname: str, objects: Dict, warning: "Type[Warning]"
-                 ) -> None:
+    def __init__(self, module: Any, modname: str,
+                 objects: Dict[str, object],
+                 warning: Type[Warning],
+                 names: Dict[str, str]) -> None:
         self._module = module
         self._modname = modname
         self._objects = objects
         self._warning = warning
+        self._names = names
 
     def __getattr__(self, name: str) -> Any:
-        if name in self._objects:
-            warnings.warn("%s.%s is deprecated. Check CHANGES for Sphinx "
-                          "API modifications." % (self._modname, name),
-                          self._warning, stacklevel=3)
-            return self._objects[name]
+        if name not in self._objects:
+            return getattr(self._module, name)
 
-        return getattr(self._module, name)
+        canonical_name = self._names.get(name, None)
+        if canonical_name is not None:
+            warnings.warn(
+                "The alias '{}.{}' is deprecated, use '{}' instead. Check CHANGES for "
+                "Sphinx API modifications.".format(self._modname, name, canonical_name),
+                self._warning, stacklevel=3)
+        else:
+            warnings.warn("{}.{} is deprecated. Check CHANGES for Sphinx "
+                          "API modifications.".format(self._modname, name),
+                          self._warning, stacklevel=3)
+        return self._objects[name]
 
 
 class DeprecatedDict(dict):
     """A deprecated dict which warns on each access."""
 
-    def __init__(self, data: Dict, message: str, warning: "Type[Warning]") -> None:
+    def __init__(self, data: Dict, message: str, warning: Type[Warning]) -> None:
         self.message = message
         self.warning = warning
         super().__init__(data)
