@@ -4,43 +4,48 @@
 
     Docutils transforms used by Sphinx when reading documents.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from os import path
 from textwrap import indent
-from typing import Any, TypeVar
+from typing import TYPE_CHECKING, Any, Dict, List, Tuple, Type, TypeVar
 
 from docutils import nodes
 from docutils.io import StringInput
+from docutils.nodes import Element
 from docutils.utils import relative_path
 
 from sphinx import addnodes
+from sphinx.config import Config
 from sphinx.domains.std import make_glossary_term, split_term_classifiers
-from sphinx.locale import __, init as init_locale
+from sphinx.locale import __
+from sphinx.locale import init as init_locale
 from sphinx.transforms import SphinxTransform
-from sphinx.util import split_index_msg, logging
+from sphinx.util import get_filetype, logging, split_index_msg
 from sphinx.util.i18n import docname_to_domain
-from sphinx.util.nodes import (
-    LITERAL_TYPE_NODES, IMAGE_TYPE_NODES, NodeMatcher,
-    extract_messages, is_pending_meta, traverse_translatable_index,
-)
+from sphinx.util.nodes import (IMAGE_TYPE_NODES, LITERAL_TYPE_NODES, NodeMatcher,
+                               extract_messages, is_pending_meta, traverse_translatable_index)
+
+if TYPE_CHECKING:
+    from sphinx.application import Sphinx
 
-if False:
-    # For type annotation
-    from typing import Dict, List, Tuple  # NOQA
-    from typing import Type  # for python3.5.1
-    from sphinx.application import Sphinx  # NOQA
-    from sphinx.config import Config  # NOQA
 
 logger = logging.getLogger(__name__)
 
+# The attributes not copied to the translated node
+#
+# * refexplict: For allow to give (or not to give) an explicit title
+#               to the pending_xref on translation
+EXCLUDED_PENDING_XREF_ATTRIBUTES = ('refexplicit',)
+
+
 N = TypeVar('N', bound=nodes.Node)
 
 
-def publish_msgstr(app, source, source_path, source_line, config, settings):
-    # type: (Sphinx, str, str, int, Config, Any) -> nodes.Element
+def publish_msgstr(app: "Sphinx", source: str, source_path: str, source_line: int,
+                   config: Config, settings: Any) -> Element:
     """Publish msgstr (single line) into docutils document
 
     :param sphinx.application.Sphinx app: sphinx application
@@ -52,20 +57,29 @@ def publish_msgstr(app, source, source_path, source_line, config, settings):
     :return: document
     :rtype: docutils.nodes.document
     """
-    from sphinx.io import SphinxI18nReader
-    reader = SphinxI18nReader(app)
-    parser = app.registry.create_source_parser(app, 'restructuredtext')
-    doc = reader.read(
-        source=StringInput(source=source,
-                           source_path="%s:%s:<translated>" % (source_path, source_line)),
-        parser=parser,
-        settings=settings,
-    )
     try:
-        doc = doc[0]  # type: ignore
-    except IndexError:  # empty node
-        pass
-    return doc
+        # clear rst_prolog temporarily
+        rst_prolog = config.rst_prolog
+        config.rst_prolog = None  # type: ignore
+
+        from sphinx.io import SphinxI18nReader
+        reader = SphinxI18nReader()
+        reader.setup(app)
+        filetype = get_filetype(config.source_suffix, source_path)
+        parser = app.registry.create_source_parser(app, filetype)
+        doc = reader.read(
+            source=StringInput(source=source,
+                               source_path="%s:%s:<translated>" % (source_path, source_line)),
+            parser=parser,
+            settings=settings,
+        )
+        try:
+            doc = doc[0]  # type: ignore
+        except IndexError:  # empty node
+            pass
+        return doc
+    finally:
+        config.rst_prolog = rst_prolog  # type: ignore
 
 
 class PreserveTranslatableMessages(SphinxTransform):
@@ -74,8 +88,7 @@ class PreserveTranslatableMessages(SphinxTransform):
     """
     default_priority = 10  # this MUST be invoked before Locale transform
 
-    def apply(self, **kwargs):
-        # type: (Any) -> None
+    def apply(self, **kwargs: Any) -> None:
         for node in self.document.traverse(addnodes.translatable):
             node.preserve_original_messages()
 
@@ -86,8 +99,7 @@ class Locale(SphinxTransform):
     """
     default_priority = 20
 
-    def apply(self, **kwargs):
-        # type: (Any) -> None
+    def apply(self, **kwargs: Any) -> None:
         settings, source = self.document.settings, self.document['source']
         msgstr = ''
 
@@ -134,7 +146,7 @@ class Locale(SphinxTransform):
             processed = False  # skip flag
 
             # update title(section) target name-id mapping
-            if isinstance(node, nodes.title):
+            if isinstance(node, nodes.title) and isinstance(node.parent, nodes.section):
                 section_node = node.parent
                 new_name = nodes.fully_normalize_name(patch.astext())
                 old_name = nodes.fully_normalize_name(node.astext())
@@ -194,18 +206,13 @@ class Locale(SphinxTransform):
 
             # glossary terms update refid
             if isinstance(node, nodes.term):
-                gloss_entries = self.env.temp_data.setdefault('gloss_entries', set())
-                for _id in node['names']:
-                    if _id in gloss_entries:
-                        gloss_entries.remove(_id)
-
+                for _id in node['ids']:
                     parts = split_term_classifiers(msgstr)
                     patch = publish_msgstr(self.app, parts[0], source,
                                            node.line, self.config, settings)
                     patch = make_glossary_term(self.env, patch, parts[1],
-                                               source, node.line, _id)
-                    node['ids'] = patch['ids']
-                    node['names'] = patch['names']
+                                               source, node.line, _id,
+                                               self.document)
                     processed = True
 
             # update leaves with processed nodes
@@ -235,6 +242,10 @@ class Locale(SphinxTransform):
                 node.details['nodes'][0]['content'] = msgstr
                 continue
 
+            if isinstance(node, nodes.image) and node.get('alt') == msg:
+                node['alt'] = msgstr
+                continue
+
             # Avoid "Literal block expected; none found." warnings.
             # If msgstr ends with '::' then it cause warning message at
             # parser.parse() processing.
@@ -253,7 +264,7 @@ class Locale(SphinxTransform):
             # see: http://docutils.sourceforge.net/docs/ref/doctree.html#structural-subelements
             if isinstance(node, nodes.title):
                 # This generates: <section ...><title>msgstr</title></section>
-                msgstr = msgstr + '\n' + '-' * len(msgstr) * 2
+                msgstr = msgstr + '\n' + '=' * len(msgstr) * 2
 
             patch = publish_msgstr(self.app, msgstr, source,
                                    node.line, self.config, settings)
@@ -264,10 +275,10 @@ class Locale(SphinxTransform):
                 patch = patch.next_node()
 
             # ignore unexpected markups in translation message
-            unexpected = (
+            unexpected: Tuple[Type[Element], ...] = (
                 nodes.paragraph,    # expected form of translation
                 nodes.title         # generated by above "Subelements phase2"
-            )  # type: Tuple[Type[nodes.Element], ...]
+            )
 
             # following types are expected if
             # config.gettext_additional_targets is configured
@@ -278,16 +289,15 @@ class Locale(SphinxTransform):
                 continue  # skip
 
             # auto-numbered foot note reference should use original 'ids'.
-            def list_replace_or_append(lst, old, new):
-                # type: (List[N], N, N) -> None
+            def list_replace_or_append(lst: List[N], old: N, new: N) -> None:
                 if old in lst:
                     lst[lst.index(old)] = new
                 else:
                     lst.append(new)
 
             is_autofootnote_ref = NodeMatcher(nodes.footnote_reference, auto=Any)
-            old_foot_refs = node.traverse(is_autofootnote_ref)  # type: List[nodes.footnote_reference]  # NOQA
-            new_foot_refs = patch.traverse(is_autofootnote_ref)  # type: List[nodes.footnote_reference]  # NOQA
+            old_foot_refs: List[nodes.footnote_reference] = node.traverse(is_autofootnote_ref)
+            new_foot_refs: List[nodes.footnote_reference] = patch.traverse(is_autofootnote_ref)
             if len(old_foot_refs) != len(new_foot_refs):
                 old_foot_ref_rawsources = [ref.rawsource for ref in old_foot_refs]
                 new_foot_ref_rawsources = [ref.rawsource for ref in new_foot_refs]
@@ -295,13 +305,14 @@ class Locale(SphinxTransform):
                                   ' original: {0}, translated: {1}')
                                .format(old_foot_ref_rawsources, new_foot_ref_rawsources),
                                location=node)
-            old_foot_namerefs = {}  # type: Dict[str, List[nodes.footnote_reference]]
+            old_foot_namerefs: Dict[str, List[nodes.footnote_reference]] = {}
             for r in old_foot_refs:
                 old_foot_namerefs.setdefault(r.get('refname'), []).append(r)
             for newf in new_foot_refs:
                 refname = newf.get('refname')
                 refs = old_foot_namerefs.get(refname, [])
                 if not refs:
+                    newf.parent.remove(newf)
                     continue
 
                 oldf = refs.pop(0)
@@ -328,8 +339,8 @@ class Locale(SphinxTransform):
             # * use translated refname for section refname.
             # * inline reference "`Python <...>`_" has no 'refname'.
             is_refnamed_ref = NodeMatcher(nodes.reference, refname=Any)
-            old_refs = node.traverse(is_refnamed_ref)  # type: List[nodes.reference]
-            new_refs = patch.traverse(is_refnamed_ref)  # type: List[nodes.reference]
+            old_refs: List[nodes.reference] = node.traverse(is_refnamed_ref)
+            new_refs: List[nodes.reference] = patch.traverse(is_refnamed_ref)
             if len(old_refs) != len(new_refs):
                 old_ref_rawsources = [ref.rawsource for ref in old_refs]
                 new_ref_rawsources = [ref.rawsource for ref in new_refs]
@@ -357,7 +368,7 @@ class Locale(SphinxTransform):
             is_refnamed_footnote_ref = NodeMatcher(nodes.footnote_reference, refname=Any)
             old_foot_refs = node.traverse(is_refnamed_footnote_ref)
             new_foot_refs = patch.traverse(is_refnamed_footnote_ref)
-            refname_ids_map = {}  # type: Dict[str, List[str]]
+            refname_ids_map: Dict[str, List[str]] = {}
             if len(old_foot_refs) != len(new_foot_refs):
                 old_foot_ref_rawsources = [ref.rawsource for ref in old_foot_refs]
                 new_foot_ref_rawsources = [ref.rawsource for ref in new_foot_refs]
@@ -374,8 +385,8 @@ class Locale(SphinxTransform):
 
             # citation should use original 'ids'.
             is_citation_ref = NodeMatcher(nodes.citation_reference, refname=Any)
-            old_cite_refs = node.traverse(is_citation_ref)  # type: List[nodes.citation_reference]  # NOQA
-            new_cite_refs = patch.traverse(is_citation_ref)  # type: List[nodes.citation_reference]  # NOQA
+            old_cite_refs: List[nodes.citation_reference] = node.traverse(is_citation_ref)
+            new_cite_refs: List[nodes.citation_reference] = patch.traverse(is_citation_ref)
             refname_ids_map = {}
             if len(old_cite_refs) != len(new_cite_refs):
                 old_cite_ref_rawsources = [ref.rawsource for ref in old_cite_refs]
@@ -405,8 +416,7 @@ class Locale(SphinxTransform):
                                .format(old_xref_rawsources, new_xref_rawsources),
                                location=node)
 
-            def get_ref_key(node):
-                # type: (addnodes.pending_xref) -> Tuple[str, str, str]
+            def get_ref_key(node: addnodes.pending_xref) -> Tuple[str, str, str]:
                 case = node["refdomain"], node["reftype"]
                 if case == ('std', 'term'):
                     return None
@@ -425,11 +435,8 @@ class Locale(SphinxTransform):
                 # Copy attributes to keep original node behavior. Especially
                 # copying 'reftarget', 'py:module', 'py:class' are needed.
                 for k, v in xref_reftarget_map.get(key, {}).items():
-                    # Note: This implementation overwrite all attributes.
-                    # if some attributes `k` should not be overwritten,
-                    # you should provide exclude list as:
-                    # `if k not in EXCLUDE_LIST: new[k] = v`
-                    new[k] = v
+                    if k not in EXCLUDED_PENDING_XREF_ATTRIBUTES:
+                        new[k] = v
 
             # update leaves
             for child in patch.children:
@@ -440,15 +447,16 @@ class Locale(SphinxTransform):
             if isinstance(node, LITERAL_TYPE_NODES):
                 node.rawsource = node.astext()
 
-            if isinstance(node, IMAGE_TYPE_NODES):
-                node.update_all_atts(patch)
+            if isinstance(node, nodes.image) and node.get('alt') != msg:
+                node['uri'] = patch['uri']
+                continue  # do not mark translated
 
             node['translated'] = True  # to avoid double translation
 
         if 'index' in self.config.gettext_additional_targets:
             # Extract and translate messages for index entries.
             for node, entries in traverse_translatable_index(self.document):
-                new_entries = []   # type: List[Tuple[str, str, str, str, str]]
+                new_entries: List[Tuple[str, str, str, str, str]] = []
                 for type, msg, tid, main, key_ in entries:
                     msg_parts = split_index_msg(type, msg)
                     msgstr_parts = []
@@ -464,7 +472,7 @@ class Locale(SphinxTransform):
                 node['entries'] = new_entries
 
         # remove translated attribute that is used for avoiding double translation.
-        for translated in self.document.traverse(NodeMatcher(translated=Any)):  # type: nodes.Element  # NOQA
+        for translated in self.document.traverse(NodeMatcher(translated=Any)):  # type: Element  # NOQA
             translated.delattr('translated')
 
 
@@ -474,8 +482,7 @@ class RemoveTranslatableInline(SphinxTransform):
     """
     default_priority = 999
 
-    def apply(self, **kwargs):
-        # type: (Any) -> None
+    def apply(self, **kwargs: Any) -> None:
         from sphinx.builders.gettext import MessageCatalogBuilder
         if isinstance(self.app.builder, MessageCatalogBuilder):
             return
@@ -486,8 +493,7 @@ class RemoveTranslatableInline(SphinxTransform):
             inline.parent += inline.children
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[str, Any]
+def setup(app: "Sphinx") -> Dict[str, Any]:
     app.add_transform(PreserveTranslatableMessages)
     app.add_transform(Locale)
     app.add_transform(RemoveTranslatableInline)
