@@ -4,7 +4,7 @@
 
     Utilities for docstring processing.
 
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -17,14 +17,13 @@ from docutils.parsers.rst.states import Body
 
 from sphinx.deprecation import RemovedInSphinx50Warning
 
-
 field_list_item_re = re.compile(Body.patterns['field_marker'])
 
 
 def extract_metadata(s: str) -> Dict[str, str]:
     """Extract metadata from docstring."""
     in_other_element = False
-    metadata = {}  # type: Dict[str, str]
+    metadata: Dict[str, str] = {}
 
     if not s:
         return metadata
@@ -57,7 +56,7 @@ def prepare_docstring(s: str, ignore: int = None, tabsize: int = 8) -> List[str]
     if ignore is None:
         ignore = 1
     else:
-        warnings.warn("The 'ignore' argument to parepare_docstring() is deprecated.",
+        warnings.warn("The 'ignore' argument to prepare_docstring() is deprecated.",
                       RemovedInSphinx50Warning, stacklevel=2)
 
     lines = s.expandtabs(tabsize).splitlines()
