@@ -10,15 +10,14 @@
 
 import os
 import posixpath
-from typing import Callable, Dict
+from typing import TYPE_CHECKING, Callable, Dict
 
 from docutils.utils import relative_path
 
 from sphinx.util.osutil import copyfile, ensuredir
 from sphinx.util.typing import PathMatcher
 
-if False:
-    # For type annotation
+if TYPE_CHECKING:
     from sphinx.util.template import BaseRenderer
 
 
